<?php

namespace SupNext\Controllers\QuoteSheetGrid;

use SupNext\Util\Util;
use SupNext\Services\QuoteSheetGrid\AddRegionService as Service;
use Teknisa\Libs\Service\CRUD as CrudService;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class AddRegionController {
  private $util;
  private $service;
  private $crudService;

  public function __construct(Util $util, Service $service, CrudService $crudService) {
    $this->util        = $util;
    $this->service     = $service;
    $this->crudService = $crudService;
  }

 public function addRegion(Request $request, Response $response) {
    try{
      $data = $request->getParameters();

      $regionExist = $this->verifyRegionExist($data);

      if(!empty($regionExist)) return $response->setError(new Error("Região já cadastrada.", 400));

      $this->validateRegionSupplier($data, $response);

      $response->addDataSet(new DataSet("addRegion", ["message" => "Inclusão da região realizada com sucesso.", "status_code" => 200]));
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao adicionar região: " . $exception->getMessage(), 500);
    }
  }

  private function verifyRegionExist(array $data): array {
    $conditions = $this->util->createConditions([
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "CDFILIAL"    => $data["CDFILIAL"],
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "CDREGIAO"    => $data["CDREGIAO"],
      "NRORG"       => $data["NRORG"]
    ]);

    $regionExist = $this->crudService
      ->getFilteredDataSet("verifyRegionExist", $conditions)
      ->getRows();

    return $regionExist;
  }

  private function validateRegionSupplier(array $data, Response $response) {
    $suppliers = $this->getSupplierData($data);

    $suppliersMeet = [];
    $qualifiedSuppiers = 0;

    for ($i = 0; $i < count($suppliers); $i++) {
      $conditionsArray = [
        "CDFORNECED"   => $data["CDFORNECED"],
        "NRORG"        => $data["NRORG"],
        "IDULTIDTVALQ" => $data["IDULTIDTVALQ"],
        "DTCOLEPREC"   => $data["DTCOLEPREC"],
        "IDULTIDTVALQ" => $data["IDULTIDTVALQ"],
        "DTCOLEPREC"   => $data["DTCOLEPREC"],
        "CDFORNECED"   => $suppliers[$i]["CDFORNECED"],
        "NRORG"        => $data["NRORG"]
      ];
      $supplierSituation = $this->util->filteredDataSet($conditionsArray, "verifySupplierSituation");
      $conditionsArray = [
        "CDFORNECED"  => $suppliers[$i]["CDFORNECED"],
        "CDREGIAO"    => $data["CDREGIAO"],
        "NRORG"       => $data["NRORG"]
      ];
      $regionCode = $this->util->filteredDataSet($conditionsArray, "verifyRegionSupplier");

      $qualifiedSuppiers +=
        !empty($supplierSituation[$i]["IDSITUCADA"]) ? 1 : 0;

      if (!empty($regionCode)) return $response->setError(new Error("Fornecedor " . $suppliers[$i]["NMFANTFORN"] . " não atende às regiões da cotação.", 400));

      array_push($suppliersMeet, $suppliers[$i]);
    }

    if ($qualifiedSuppiers == 0) return $response->setError(new Error("sup::NO_ACTIVE_QUALIFIED_SUPPLIER_FOR_QUOTE", 400));

    if (empty($suppliersMeet)) return $response->setError(new Error("sup::SUPPLIERS_NOT_SERVING_REGIONS", 400));

    foreach ($suppliersMeet as $supplier) {
      $conditionsArray = [
        "CDFILIAL"    => $data["CDFILIAL"],
        "NRCOLEPRECO" => $data["NRCOLEPRECO"],
        "NRORG"       => $data["NRORG"],
        "CDFORNECED"  => $supplier["CDFORNECED"]
      ];
      $products = $this->util->filteredDataSet($conditionsArray, "getProductsM");

      if (empty($products)) return $response->setError(new Error("Fornecedor " . $supplier["NMFANTFORN"] . " não cadastrado para o produto da cotação.", 400));

      foreach ($products as $product) {
        $conditionsArray = [
          "CDPRODINI"  => $product["CDPRODUTO"],
          "NRORG"      => $data["NRORG"],
          "CDFORNECED" => $supplier["CDFORNECED"],
          "NRORG"      => $data["NRORG"],
          "CDFILIAL"   => $data["CDFILIAL"],
          "DTCOL"      => $product["DTCOLEPREC"],
          "DTCOL"      => $product["DTCOLEPREC"]
        ];
        $productsSupplier = $this->util->filteredDataSet($conditionsArray, "verifyProductSupplier")[0];

        if (empty($productsSupplier)) return $response->setError(new Error("Fornecedor " . $supplier["NMFANTFORN"] . " desqualificado para o produto da cotação.", 400));

        $unitPrice = 0;
        $conditionsArray = [
          "CDFILIAL"    => $data["CDFILIAL"],
          "NRCOLEPRECO" => $data["NRCOLEPRECO"],
          "CDREGIAO"    => $data["CDREGIAOPREC"],
          "CDFORNECED"  => $data["CDFORNECED"],
          "CDPRODUTO"   => $product["CDPRODUTO"],
          "NRORG"       => $data["NRORG"]
        ];

        if (!empty($data["CDREGIAOPREC"])) $unitPrice = $this->util->filteredDataSet($conditionsArray, "getUnitPrice");

        $conditionsArray = [
          "CDFILIAL"    => $data["CDFILIAL"],
          "NRCOLEPRECO" => $data["NRCOLEPRECO"],
          "NRORG"       => $data["NRORG"]
        ];

        $priceList = $this->util->filteredDataSet($conditionsArray, "getPriceList")[0];

        $parameters = $this->processAddRegionData($data, $supplier, $product, $priceList, $unitPrice);

        if ($data['IDUTINDICECOL'] == 'S') {
          $this->service->insertItemColeWithIndex($parameters);
        } else {
          $this->service->insertItemColeWithoutIndex($parameters);
        }

        $this->executeProcedureItemcol($data, $supplier, $product);
        $this->processInsertFormPgtoData($data, $supplier, $product);
      }
    }
  }

  private function getSupplierData(array $data): array {
    $conditionsArray = [
      "CDFILIAL"    => $data["CDFILIAL"],
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "NRORG"       => $data["NRORG"]
    ];

    if (is_null($data["CDFORNECED"]) && is_null($data["NRINSJURFORN"])) return $this->util->filteredDataSet($conditionsArray, "supplierQuotation");

    return [[
      "NRINSJURFORN" => $data["NRINSJURFORN"],
      "NMFANTFORN"   => $data["NMFANTFORN"],
      "CDFORNECED"   => $data["CDFORNECED"]
    ]];
  }

  private function processAddRegionData(array $data, array $supplier, array $product, array $priceList, $unitPrice): array {
    $countUnitPrice = 0;

    if (is_array($unitPrice)) $countUnitPrice = count($unitPrice);

    $parameters = [
      "NRSEQPRELIST"    => $priceList["NRSEQPRELIST"],
      "IDITVALDIFLISTA" => $priceList["IDITVALDIFLISTA"] ? $priceList["IDITVALDIFLISTA"] : "N",
      "IDPRECALT"       => $priceList["IDPRECALT"]       ? $priceList["IDPRECALT"]       : "N",
      "CDFILIAL"        => $data["CDFILIAL"],
      "NRCOLEPRECO"     => $data["NRCOLEPRECO"],
      "CDPRODUTO"       => $product["CDPRODUTO"],
      "CDMARCA"         => $product["CDMARCA"],
      "CDFORNECED"      => $supplier["CDFORNECED"],
      "CDREGIAO"        => $data["CDREGIAO"],
      "VRPRECUNIITM"    => $countUnitPrice == 0 ? $countUnitPrice : $unitPrice[0]["VRPRECUNIITM"],
      "VRINDICEITMCOL"  => $countUnitPrice == 0 ? $countUnitPrice : $unitPrice[0]["VRINDICEITMCOL"],
      "CDOPERINCL"      => $data["CDOPERINCL"],
      "IDATUREG"        => null,
      "IDATUFORN"       => null,
      "NRORG"           => $data["NRORG"],
    ];

    return $parameters;
  }

  public function executeProcedureItemcol(array $data, ?array $supplier = null, ?array $product = null): void {
    $conditionsArray = [
      "CDFILIAL"  => $data["CDFILIAL"],
      "CDPRODUTO" => $product["CDPRODUTO"] ?? $data["CDPRODUTO"],
      "NRORG"     => $data["NRORG"]
    ];

    $companyCode = $this->util->filteredDataSet($conditionsArray, "getEmitterAF");

    $conditionsArray = [
      "CDFILIAL"  => $data["CDFILIAL"],
      "NRORG"     => $data["NRORG"]
    ];

    if (empty($companyCode) || $companyCode[0]["CDEMPRESA"] == null) $companyCode = $this->util->filteredDataSet($conditionsArray, "getEmitterDefault");

    if (empty($companyCode) || $companyCode[0]["CDEMPRESA"] == null) $companyCode = $this->util->filteredDataSet($conditionsArray, "getCompanyBranch");

    $conditionsArray = [
      "CDEMPRESA" => $companyCode[0]["CDEMPRESA"],
      "NRORG"     => $data["NRORG"]
    ];

    $companyInfo = $this->util->filteredDataSet($conditionsArray, "getCompanyInfo");

    $parameters = $this->processImpostoItemcolData($data, $supplier, $product, $companyInfo);

    $this->service->procImpostoItemcol($parameters);
  }

  private function processImpostoItemcolData(array $data, ?array $supplier = null, ?array $product = null, array $companyInfo){
    $parameters = [
      "CDEMPRESA"   => $companyInfo[0]["CDEMPRESA"],
      "CDINSCESTA"  => $companyInfo[0]["CDINSCESTA"],
      "CDFORNECED"  => $supplier["CDFORNECED"] ?? $data["CDFORNECED"],
      "CDFILCENT"   => $data["CDFILIAL"],
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "CDFORNECED"  => $supplier["CDFORNECED"] ?? $data["CDFORNECED"],
      "CDREGIAO"    => $data["CDREGIAO"],
      "CDPRODUTO"   => $product["CDPRODUTO"] ?? $data["CDPRODUTO"],
      "CDMARCA"     => $product["CDMARCA"] ?? $data["CDMARCA"],
      "DTCOLEPREC"  => $data["DTCOLEPREC"]
    ];

    return $parameters;
  }

  public function processInsertFormPgtoData(array $data, ?array $supplier = null, ?array $product = null): void {
    $conditionsArray = [
      "CDFILIAL"    => $data["CDFILIAL"],
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "CDFORNECED"  => $supplier["CDFORNECED"] ?? $data["CDFORNECED"],
      "CDPRODUTO"   => $product["CDPRODUTO"] ?? $data["CDPRODUTO"],
      "CDMARCA"     => $product["CDMARCA"] ?? $data["CDMARCA"],
      "CDREGIAO"    => $data["CDREGIAO"],
      "NRORG"       => $data["NRORG"]
    ];

    $contPzoCot = $this->util->filteredDataSet($conditionsArray, "getContPzoCot");

    if ($contPzoCot[0]["CTPZOS"] == 0) {
      $conditionsArray = [
        "CDFORNECED"  => $supplier["CDFORNECED"] ?? $data["CDFORNECED"],
        "NRORG"       => $data["NRORG"],
        "CDFORNECED"  => $supplier["CDFORNECED"] ?? $data["CDFORNECED"],
        "CDPRODUTO"   => $product["CDPRODUTO"] ?? $data["CDFORNECED"],
        "NRORG"       => $data["NRORG"]
      ];

      $paymentProdSup = $this->util->filteredDataSet($conditionsArray, "getPgtoProdForn");

      if (empty($paymentProdSup) || $paymentProdSup[0]["IDTPFORPGPRF"] == "") {
        $conditionsArray = [
          "CDFORNECED"  => $supplier["CDFORNECED"] ?? $data["CDFORNECED"],
          "NRORG"       => $data["NRORG"]
        ];

        $paymentSupplier = $this->util->filteredDataSet($conditionsArray, "getPaymentSupplier");

        $this->service->updatePaymentTerm($data, $supplier, $product, $paymentSupplier);

        if ($paymentSupplier[0]["IDTPFORPGTFO"] == "V" ) {
          $this->service->deleteSupplierRegister($data, $supplier, $product);

          $conditionsArray = [
            "CDFILIAL"    => $data["CDFILIAL"],
            "NRCOLEPRECO" => $data["NRCOLEPRECO"],
            "CDFORNECED"  => $supplier["CDFORNECED"] ?? $data["CDFORNECED"],
            "CDPRODUTO"   => $product["CDPRODUTO"] ?? $data["CDPRODUTO"],
            "CDMARCA"     => $product["CDMARCA"] ?? $data["CDMARCA"],
            "CDREGIAO"    => $data["CDREGIAO"],
            "NRORG"       => $data["NRORG"]
          ];

          $installments = $this->util->filteredDataSet($conditionsArray, "getInstallments");

          if (!empty($installments)) {
            foreach ($installments as $installment) {
              $this->service->insertSupplier($data, $supplier, $product, $installment);
            }
          }

          $this->service->forcTrigger($data, $supplier, $product);
        }
      } else {
        $this->service->updatePaymentTerm($data, $supplier, $product, $paymentProdSup);

        if ($paymentProdSup[0]["IDTPFORPGPRF"] == "V") $this->service->insertSupplierProduct($data, $supplier, $product);

        $this->service->forcTrigger($data, $supplier, $product);
      }
    }
  }
}