import { TekGrid } from '@zeedhi/teknisa-components-common';
import {
  Date as DateInput, Form, Select, LoadingService, ModalService,
} from '@zeedhi/common';
import {
  Metadata, dayjs, Http,
} from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';
import { AppState } from '@/controllers/AppState';
import { ValidateBuyerController } from '@/controllers/modals/quoteSheetGrid/ValidateBuyerController';
import type { ISupplierFilter, IFormQuotationInclusion } from '@/controllers/modals/interfaces/QuoteSheetGrid';

export class AddQuotationController {
  private utilities!: Utilities;

  private appState: AppState;

  constructor() {
    this.utilities = new Utilities();
    this.appState = AppState.getInstance();
  }

  public get form(): Form {
    return Metadata.getInstance<Form>('addQuoteForm');
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/addQuotationModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('addQuotationModal')?.destroy();
  }

  public openModal(): void {
    ModalService.getModal('addQuotationModal').show();
  }

  public closeModal(): void {
    ModalService.getModal('addQuotationModal').hide();
  }

  public get filterBuyer(): { P_CDOPERADOR: string } {
    return { P_CDOPERADOR: this.appState.CDOPERADOR };
  }

  public onChangeComprador(): boolean {
    if (!this.form.value.quoteOperator) return false;

    if (this.form.value.quoteOperator.IDEXSENHCMPOPER === 'S') {
      ValidateBuyerController.openModal();
      ValidateBuyerController.setUser();
    }

    return true;
  }

  public get filterSupplier(): ISupplierFilter {
    return {
      P_NRORG: this.appState.NRORG,
      P_IDULTIDTVALQ: 'T',
      P_DTCOL: this.form.value.quoteCollectionDate,
    };
  }

  public get filterPurchaseNumber(): { P_NRORG: string } {
    return {
      P_NRORG: this.appState.NRORG,
    };
  }

  public getCollectionDate(): void {
    const date = Metadata.getInstance<DateInput>('quoteCollectionDate');
    date.value = dayjs();
  }

  public getValidityDate(): void {
    const date = Metadata.getInstance<DateInput>('quoteValidity');
    date.value = [dayjs(), dayjs()];
  }

  public validatePurchase(): void {
    if (
      this.form.value.quotePurchaseNumber
      && this.form.value.quoteOperator.CDOPERADOR
      !== this.form.value.quotePurchaseNumber.CDCOMPRADOR
    ) {
      this.utilities
        .showWarningDialog(
          'Operação bloqueada. Compra pertence a um comprador diferente do comprador da cotação.',
        );
      this.form.value.quotePurchaseNumber = null;
    }
  }

  public async includeQuote(): Promise<void> {
    const isValid = this.validateForm();

    if (!isValid) return;

    const params = this.processParams();

    LoadingService.show();

    try {
      const { data } = await Http.post('/addQuotation', params);

      if (data.error) {
        this.utilities.showErrorDialog(data.error);
        LoadingService.hide();

        return;
      }

      LoadingService.hide();

      this.utilities.showSuccessDialog('sup::SUCCESS');

      this.closeModal();
      this.resetForm();

      const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
      quoteGrid.reload();

      return;
    } catch (error) {
      LoadingService.hide();
      this.utilities.showErrorDialog('sup::ERROR');
    }
  }

  public validateProducts(): void {
    const initProduct = this.form.getChildInstance<Select>('quoteInitProduct');
    const endProduct = this.form.getChildInstance<Select>('quoteEndProduct');

    if ((!initProduct.value && endProduct.value) || (initProduct.value && !endProduct.value)) {
      initProduct.label = 'Produto Inicial *';
      endProduct.label = 'Produto Final *';
    } else if (!initProduct.value && !endProduct.value) {
      initProduct.label = 'Produto Inicial';
      endProduct.label = 'Produto Final';
    }
  }

  private validateForm(): boolean {
    const initProduct = this.form.getChildInstance<Select>('quoteInitProduct');
    const endProduct = this.form.getChildInstance<Select>('quoteEndProduct');

    if (!this.form.validate()) return false;

    if (!initProduct.value && !endProduct.value) return true;

    if ((!initProduct.value && endProduct.value) || (initProduct.value && !endProduct.value)) {
      this.utilities.showErrorDialog('sup::BOTH_FIELDS_REQUIRED_OR_EMPTY');
      return false;
    }

    const validateProduct = this.utilities.validateProduct(initProduct.value, endProduct.value);

    if (!validateProduct.isValid) {
      this.utilities.showErrorDialog(validateProduct.message);
      return false;
    }

    return true;
  }

  private processParams(): IFormQuotationInclusion {
    const NRSEMAINIC = this.form.value.quoteValidity[0].split('/').reverse().join('/');
    const NRSEMAFINAL = this.form.value.quoteValidity[1].split('/').reverse().join('/');

    return {
      CDOPERADOR: this.form.value.quoteOperator.CDOPERADOR,
      DTCOLEPREC: this.form.value.quoteCollectionDate,
      CDREGIAO: this.form.value.quoteRegion,
      CDFORNECED: this.form.value.quoteSupplier,
      CDMARCA: this.form.value.CDMARCA,
      DTINIVALCOL: this.form.value.quoteValidity[0],
      DTFINVALCOL: this.form.value.quoteValidity[1],
      NRSEMAINIC: this.utilities.getWeekNumber(NRSEMAINIC),
      NRSEMAFINAL: this.utilities.getWeekNumber(NRSEMAFINAL),
      IDTIPOCOTACAO: this.form.value.quoteTypeSituation,
      CDPRODINI: this.form.value.quoteInitProduct,
      CDPRODFIN: this.form.value.quoteEndProduct,
      CDVERIF: this.form.value.CDVERIF,
      IDCARGOBS: this.form.value.IDCARGOBS,
      IDCARGULTC: this.form.value.IDCARGULTC,
      IDDISMONTLIS: this.form.value.IDDISMONTLIS,
      NRCOMPRA: this.form.value.quotePurchaseNumber?.NRCOMPRA ?? null,
      DSOBSCOTPREC: this.form.value.DSOBSCOTPREC,
      CDOPERINCL: this.appState.CDOPERADOR,
    };
  }

  private resetForm(): void {
    this.form.clearForm();
    this.getCollectionDate();
    this.getValidityDate();
    this.form.value.quoteTypeSituation = 'A';
    this.form.value.IDCARGOBS = 'S';
    this.form.value.IDCARGULTC = 'N';
    this.form.value.IDDISMONTLIS = 'S';
  }
}
