import { TekGrid } from '@zeedhi/teknisa-components-common';
import {
  ModalService, Form, LoadingService,
} from '@zeedhi/common';
import { Metadata, Http } from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';
import { AppState } from '@/controllers/AppState';
import type { IFilterQuoteSupplier, IFilterLoadRegionPrices } from '@/controllers/modals/interfaces/QuoteSheetGrid';

export class AddSupplierController {
  private utilities!: Utilities;

  private appState: AppState;

  constructor() {
    this.utilities = new Utilities();
    this.appState = AppState.getInstance();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/addSupplierModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('addSupplierModal')?.destroy();
  }

  public openModal(): void {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const row = grid.datasource.currentRow;

    if (row.NRCOLEPRECO) {
      ModalService.getModal('addSupplierModal').show();
    } else {
      this.utilities.showWarningDialog('Selecione uma cotação para incluir um novo fornecedor.');
    }
  }

  public closeModal(): void {
    ModalService.getModal('addSupplierModal').hide();
  }

  public get filterQuoteSupplier(): IFilterQuoteSupplier {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsRegionGrid');
    const row = grid.datasource.currentRow;

    return {
      P_CDFILIAL: this.appState.CDFILIAL,
      P_NRCOLEPRECO: row.NRCOLEPRECO,
      P_NRORG: this.appState.NRORG,
    };
  }

  public get filterLoadRegionPrices():IFilterLoadRegionPrices {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const row = grid.datasource.currentRow;

    return {
      P_CDFILIAL: this.appState.CDFILIAL,
      P_NRCOLEPRECO: row.NRCOLEPRECO,
      P_NRORG: this.appState.NRORG,
    };
  }

  public async addSupplier(): Promise<void> {
    const form = Metadata.getInstance<Form>('addSupplierForm');
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const supplier = Metadata.getInstance<TekGrid>('multipleLevelsSupplierGrid');
    const isValid = form.validate();

    if (!isValid) return;

    const params = this.processData(form, grid);

    LoadingService.show();

    try {
      const { data } = await Http.post('/addSupplier', params);

      if (data.error) {
        LoadingService.hide();

        if (!Array.isArray(data.error)) {
          this.utilities.showErrorDialog(data.error);
          return;
        }

        const formattedMessages = this.formatMessages(data.error);

        this.utilities.showErrorDialog(formattedMessages, 'Erros na inclusão');

        supplier.reload();

        return;
      }

      LoadingService.hide();

      this.utilities.showSuccessDialog('sup::SUCCESS');

      this.closeModal();
      form.clearForm();
      supplier.reload();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showErrorDialog('sup::ERROR');
    }
  }

  private formatMessages(dataErrors) {
    const errors = this.filterErrors(dataErrors);
    const messages = [];

    errors.forEach((item) => {
      switch (item.DSERRO) {
        case 'FORN_N_ATENDE_REGIAO':
          messages.push(
            `Fornecedor ${item.NMRAZSOCFORN} não atende a região ${item.NMREGIAO}`,
          );
          break;
        case 'FORN_N_CAD_PROD':
          messages.push(
            `Fornecedor ${item.NMRAZSOCFORN} não cadastrado
            para o produto ${item.CDARVPROD} - ${item.NMPRODUTO} da cotação.`,
          );
          break;
        case 'FORN_N_QUAL_PROD':
          messages.push(
            `Fornecedor ${item.NMRAZSOCFORN} desqualificado
            para o produto ${item.CDARVPROD} - ${item.NMPRODUTO} da cotação.`,
          );
          break;
        default:
          break;
      }
    });

    const formattedMessages = messages.join('<br />');

    return formattedMessages;
  }

  private filterErrors(dataErrors) {
    const errorsMap = new Map();

    dataErrors.forEach((item) => {
      if (!errorsMap.has(item.CDREGIAO) && !errorsMap.has(item.DSERRO)) {
        errorsMap.set(item.CDREGIAO, item);
      }
    });

    const errors = Array.from(errorsMap.values());

    return errors;
  }

  private processData(form: Form, grid: TekGrid) {
    return {
      CDFILIAL: this.appState.CDFILIAL,
      CDFILIPROC: this.appState.CDFILCENTCMP,
      CDFORNCARG: form.value.forSupplier?.CDFORNECED ?? null,
      CDFORNECED: form.value.supplier.CDFORNECED,
      CDOPERINCL: this.appState.CDOPERADOR,
      CDREGCARG: form.value.loadRegionPricesSupplier?.CDREGIAO ?? null,
      DTCOLEPREC: grid.datasource.currentRow.DTCOLEPREC,
      IDTPIJURFORN: form.value.supplier.IDTPIJURFORN,
      NMFANTFORN: form.value.supplier.NMFANTFORN,
      NMFANTFORNCARG: form.value.forSupplier?.NMFANTFORN ?? null,
      NMRAZSOCFORN: form.value.forSupplier?.NMRAZSOCFORN ?? null,
      NMREGCARG: form.value.loadRegionPricesSupplier?.NMREGIAO,
      NRCOLEPRECO: grid.datasource.currentRow.NRCOLEPRECO,
      NRINSJURFORN: form.value.supplier.NRINSJURFORN,
      LOADALLREGION: form.value.loadAllRegion,
    };
  }
}
