import { TekGrid } from '@zeedhi/teknisa-components-common';
import {
  ModalService,
  Date as DateInput,
  Select,
  SelectMultiple,
  Checkbox,
  Form,
  LoadingService,
} from '@zeedhi/common';
import {
  Metadata, dayjs, IEventParam, Http,
} from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';
import { AppState } from '@/controllers/AppState';
import type { IReplicateQuotationParams } from '@/controllers/modals/interfaces/QuoteSheetGrid';

export class DuplicateQuotationController {
  private utilities!: Utilities;

  private appState: AppState;

  constructor() {
    this.utilities = new Utilities();
    this.appState = AppState.getInstance();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/duplicateQuotationModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('duplicateQuotationModal')?.destroy();
  }

  public openModal(): boolean {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    if (!grid.selectedRows.length) {
      this.utilities
        .showWarningDialog(
          'Selecione uma cotação para realizar a sua replicação.',
        );

      return false;
    }

    if (grid.selectedRows.length > 1) {
      this.utilities.showWarningDialog(
        'Selecione uma única cotação para realizar a sua replicação.',
      );

      return false;
    }

    ModalService.getModal('duplicateQuotationModal').show();
    setTimeout(() => {
      this.setDefaultQuotation();
    }, 100);

    return true;
  }

  public closeModal(): void {
    ModalService.getModal('duplicateQuotationModal').hide();
  }

  public get filterQuotation() {
    return {
      P_CDFILIAL: this.appState.CDFILIAL,
      P_CDCOMPRADOR: 'T',
      P_DTINIVALCOL: null,
      P_DTFINVALCOL: null,
      P_TDATAS: 'T',
      P_CDFORNECED: 'T',
      P_NRORG: this.appState.NRORG,
    };
  }

  public setRegionFieldFilter(): void {
    this.setFilterRegion();
  }

  public setDefaultQuotation(): void {
    const quotation = Metadata.getInstance<Select>('duplicate_quotation_input');
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    quotation.setValue(grid.selectedRows[0].NRCOLEPRECO);
  }

  public getDefaultCollectionDate({ component }: any): void {
    component.value = dayjs();
  }

  public changeFieldsStates({ component }: IEventParam<Checkbox>): void {
    const quotation = Metadata.getInstance<Select>('duplicate_quotation_input');
    const quoteInterval = Metadata.getInstance<DateInput>(
      'duplicate_quotation_validity',
    );
    const buyer = Metadata.getInstance<SelectMultiple>('duplicate_quotation_buyer');

    if (component.value === 'S') {
      quotation.label = `${quotation.label} *`;
      quotation.addValidation('required');
      quotation.disabled = false;

      quoteInterval.value = [null, null];
      quoteInterval.disabled = true;
      quoteInterval.label = quoteInterval.label.substring(
        0,
        quoteInterval.label.length - 2,
      );

      buyer.value = null;
      buyer.disabled = true;

      this.setFilterRegion();
    } else {
      quotation.disabled = true;
      quotation.label = quotation.label.substring(
        0,
        quotation.label.length - 2,
      );

      quotation.clearValidations();

      quoteInterval.disabled = false;
      quoteInterval.addValidation('required');
      quoteInterval.label = `${quoteInterval.label} *`;

      buyer.disabled = false;

      this.setFilterRegion(false);
    }
  }

  private setFilterRegion(useCurrentQuotation: boolean = true): void {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const region = Metadata.getInstance<any>('duplicate_quotation_region');
    const quotation = Metadata.getInstance<Select>('duplicate_quotation_input');

    if (!useCurrentQuotation) {
      region.datasource.route = '/crudN/region';
      region.datasource.setFilter({});
    } else {
      region.datasource.setFilter({
        P_CDFILIAL: grid.selectedRows[0].CDFILIAL,
        P_NRCOLEPRECO: quotation.value ?? null,
        P_NRORG: this.appState.NRORG,
      });
    }
  }

  public async duplicateQuotation(): Promise<void> {
    const form = Metadata.getInstance<Form>('duplicate_quotation_form');
    const isValid = form.validate();

    if (!isValid) return;

    LoadingService.show();

    try {
      const params = this.prepareParams();
      const { data } = await Http.post('/duplicateQuotation', params);

      if (data.error) {
        this.utilities.showErrorDialog(data.error);
        LoadingService.hide();

        return;
      }

      this.utilities.showSuccessDialog('sup::SUCCESS');

      this.closeModal();
      form.clearForm();

      const collectionDate = Metadata.getInstance<DateInput>('duplicate_quotation_collection_date');
      this.getDefaultCollectionDate({ component: collectionDate });

      const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
      quoteGrid.reload();
    } catch (error) {
      this.utilities.showErrorDialog('sup::ERROR');
    } finally {
      LoadingService.hide();
    }
  }

  private prepareParams(): IReplicateQuotationParams {
    const form = Metadata.getInstance<Form>('duplicate_quotation_form');
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const { registeredParams } = this.appState;
    const IDDUPLICA = form.value.duplicate_quotation_input ? '2' : '1';

    return {
      CONSCOTAN:
        registeredParams.IDCOTCONCLUI === 'S'
        && registeredParams.IDUTISTATCOT === 'S'
          ? 'S'
          : 'N',
      CDFILIAL: this.appState.CDFILCENTCMP,
      CDFILIPROC: this.appState.CDFILIAL,
      CDOPERINCL: this.appState.CDOPERADOR,
      NRCOLEPRECO: !form.value.duplicate_quotation_input
        ? grid.selectedRows[0].NRCOLEPRECO
        : form.value.duplicate_quotation_input,
      IDDUPLICA,
      DTINIVALCOL:
        form.value.duplicate_quotation_validity[0] && IDDUPLICA === '1'
          ? form.value.duplicate_quotation_validity[0]
          : dayjs().format('DD/MM/YYYY'),
      DTFINVALCOL:
        form.value.duplicate_quotation_validity[1] && IDDUPLICA === '1'
          ? form.value.duplicate_quotation_validity[1]
          : dayjs().format('DD/MM/YYYY'),
      DTNOVINIVAL: form.value.duplicate_quotation_price_new_validity[0],
      DTNOVFINVAL: form.value.duplicate_quotation_price_new_validity[1],
      NRSEMAINIC: form.value.dupicateQuoteInterval
        ? this.utilities.getWeekNumber(form.value.duplicate_quotation_price_new_validity[0])
        : null,
      NRSEMAFINAL: form.value.dupicateQuoteInterval
        ? this.utilities.getWeekNumber(form.value.duplicate_quotation_price_new_validity[1])
        : null,
      DTCOLEPREC: form.value.duplicate_quotation_collection_date,
      REAJUSTE: form.value.duplicate_quotation_price_adjustment ? form.value.duplicate_quotation_price_adjustment : 0,
      CDOPERADOR: form.value.duplicate_quotation_input
        ? null
        : this.appState.CDOPERADOR,
      TCDOPERADOR: this.appState.CDOPERADOR.length ? null : 'T',
      CDREGIAO: form.value.duplicate_quotation_region ? form.value.duplicate_quotation_region : null,
      CDREGIAOOL: form.value.OLRegion ? form.value.OLRegion : null,
      DUPLICAOL: 'S',
      NRORG: this.appState.NRORG,
    };
  }
}
